/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Essai de la mini-plaquette SHT-30 (sonde température humidité I2C)
  Utilise un Raspberry Pi Pico.
  Électro-Bidouilleur, Février 2023. https://www.youtube.com/Électro-Bidouilleur
  Licence de droits: Attribution 4.0 International (CC BY 4.0)  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr
*/

import time
from machine import Pin, I2C

sht30_data = list()     # A list used to collect the SHT-30 data

# Function that calculates checksum for n bytes of data and compares it with provided checksum
# The last byte of data contains the checksum to verify.
def crc8_check(data):
    crc = 0xFF
    #calculates 8-Bit checksum with given polynomial
    for byteCtr in range(0, len(data)-1):
        crc = crc ^ (data[byteCtr])
        for bit in range(8, 0, -1):
            if (crc & 0x80): crc = (crc << 1) ^ 0x131
            else: crc = (crc << 1)
    if (crc != data[len(data)-1]): return False     # 0 = checksum mismatch. Error.
    else: return True                               # 1 = checksum match. OK.


 # Read back SHT30 data
def read_sensor_data(none):
    global sht30_data
    global status_byte
    global comm_error_cntr
    
    # At I2C address 0x44, read data back from internal address 0x00, 6 bytes
    # cTemp MSB, cTemp LSB, cTemp CRC, Humididty MSB, Humidity LSB, Humidity CRC
    # sht30_data is a list
    try:
        i2c_sh30.writeto_mem(0x44, 0x24, b'\x00')               # Configure to one-shot mode, high Repeatability
        sht30_data = i2c_sh30.readfrom_mem(0x44, 0x00, 6)       # Read temperature, humidity and the checksums
        
        # Verify received checksum vs. calculated checksum for both temperature and humidity True is checksum OK
        if crc8_check(sht30_data[0:3]) and crc8_check(sht30_data[3:]):
            print("Checksum OK")
            comm_error_cntr = 0
        else:
            print("Checksum error (link problem)")
    except:
        print("Communication down with SHT30")


# Instantiate the i2c connection to the sensor
i2c_sh30 = I2C(0,scl=Pin(21),sda=Pin(20),freq=1000) # i2c 0
time.sleep(0.5)

# Main reading loop
while(1):
    read_sensor_data(None)
    # Convert the data
    cTemp = ((((sht30_data[0] * 256.0) + sht30_data[1]) * 175) / 65535.0) - 45
    fTemp = cTemp * 1.8 + 32
    humidity = 100 * (sht30_data[3] * 256 + sht30_data[4]) / 65535.0
    # Output data to screen
    print("Relative Humidity : %.2f %%RH" %humidity)
    print ("Temperature in Celsius : %.2f C" %cTemp)
    print ("Temperature in Fahrenheit : %.2f F\n" %fTemp)
    time.sleep(2)
